﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MysingleCase.Components;
using System.Collections.Specialized;

namespace MysingleCase.Models
{
    public abstract class ForumDataProvider
    {
        public ForumDataProvider()
        { 
        
        }
        public static readonly string ForumDataProviderName = "ForumDataProvider";
        #region Instance

        private static ForumDataProvider _defaultInstance = null;

        static ForumDataProvider()
        {
            CreateDefaultCommonProvider();
        }

        /// <summary>
        /// Returns an instance of the user-specified data provider class.
        /// </summary>
        /// <returns>An instance of the user-specified data provider class.  This class must inherit the
        /// CommonDataProvider interface.</returns>
        public static ForumDataProvider Instance()
        {
            return _defaultInstance;
        }

        public static ForumDataProvider Instance(Provider dataProvider)
        {
            ForumDataProvider fdp = CSCache.Get(dataProvider.Name) as ForumDataProvider;
            if (fdp == null)
            {
                fdp = DataProviders.Invoke(dataProvider) as ForumDataProvider;
                CSCache.Max(dataProvider.Name, fdp);
            }
            return fdp;
        }

        /// <summary>
        /// Creates the Default CommonDataProvider
        /// </summary>
        private static void CreateDefaultCommonProvider()
        {
            // Get the names of the providers
            //
            CSConfiguration config = CSConfiguration.GetConfig();

            // Read the configuration specific information
            // for this provider
            //
            Provider sqlForumsProvider = (Provider)config.Providers[ForumDataProviderName];

            // Read the connection string for this provider
            //

            _defaultInstance = DataProviders.CreateInstance(sqlForumsProvider) as ForumDataProvider;
        }

        #endregion

        public abstract NewsPost AddPost(NewsPost post, int userID);
        public abstract NewsPost GetPost(int postID, int userID);
        public abstract ForumSection GetForumSection(int forumID, int userID);
        public abstract User GetUser(int userID);
        public abstract User UpdateUser(User user);
    }
}
